@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-__version__ = '0.5.1'
+__version__ = '0.6.0'
 
 try:
     from pymongo import ASCENDING, DESCENDING  # NOQA
@@ -15,5 +15,9 @@ try:
         EmailField, IntField, FloatField, DecimalField, BinaryField,
         JsonField
     )
+
+    from motorengine.aggregation.base import Aggregation  # NOQA
+    from motorengine.query_builder.node import Q  # NOQA
+
 except ImportError:  # NOQA
     pass  # likely setup.py trying to import version
