@@ -5,5 +5,7 @@ from motorengine.query.base import QueryOperator
 
 
 class GreaterThanQueryOperator(QueryOperator):
-    def to_query(self):
-        return {"$gt": self.value}
+    def to_query(self, field_name, value):
+        return {
+            field_name: {"$gt": value}
+        }
